#!/usr/bin/python
# -*- coding:utf-8 -*-

"""An abstract class of scoreboard classes.
"""

from dom import *
import scorecard

def newInstance():
    return Tennins_Sportsline_2005()


class Tennins_Sportsline_2005 ( scorecard.ScoreCard ):
    def __init__( self ):
        scorecard.ScoreCard.__init__( self )
        return

    def getSportsCategory( self ):
        return "tennis"

    def getCountry( self ):
        return "World"

    def getName( self ):
        return "Tennis Tours (sportsline.com)"

    def getVersion( self ):
        return "1.0"

    def getIcon( self, team ):
        return ""

    def applyFilter( self, contents ):
        """To eliminate malformed HTML, you can set filters
        in this method"""
        return contents

    def getLanguage( self ):
        return "en"

    def getURL( self ):
        return "http://www.sportsline.com/tennis/scoreboard"

    def __get_player( self, elem ):
        #print elem
        datum = elem.get_element_at_path( 'td' )
        name = datum.get_text( '-*-' ).split( '-*-' )[ 0 ]
        games = [ name, ]
        #print "NAME", name
        while datum:
            datum = datum.next_sibling()
            if datum == None: break
            #print datum
            games.append( datum.get_text() )
            pass
        #print games
        return games

    def __get_score( self, tournament, elem ):
        row = elem.get_element_at_path( 'tr' )
        gamestat = row.get_text()
        row = row.next_sibling( 'tr' ).next_sibling( 'tr' )
        player1 = self.__get_player( row )
        #print player1
        
        row = row.next_sibling( 'tr' )
        player2 = self.__get_player( row )
        #print gamestat, player1, player2
        score = { 'gamestat'  :gamestat,
                  'player1'   :player1[ 0 ], 'player2'   :player2[ 0 ],
                  'games':max( len( player1 ), len( player2 ) ) - 1,
                  'game1'     :player1[ 1:], 'game2'     :player2[ 1:],
                  'tournament':tournament }
        #print score
        return score

    def interpreteDocument( self, document ):
        import sys, re
        elem = document.get_element_at_path( "/html/table/tr/td/table,2" )
        #print elem
        #sys.exit()
        tournament = None
        self.tables = []
        try:
            row = elem.get_element_at_path( 'tr' )
            #print row.next_sibling().get_element_at_path( 'td,3' )#next_sibling()
            #sys.exit()
            while row:
                if len( row.get_children() ) == 1: #title
                    title = row.get_text( "-*-" ).split( "-*-" )[ -1 ]
                    tournament = title#{ 'title':title, 'tabless':[] }
                    pass
                elif tournament != None:
                    scoretable = row.get_element_at_path( 'td' )
                    while scoretable != None:
                        if len( scoretable.get_children() ) > 0:
                            elem_score = scoretable.get_element_at_path( 'div/table' )
                            if elem_score == None: break
                            score = self.__get_score( tournament, elem_score )
                            #print score
                            self.tables.append( score )
                            pass
                        try:
                            scoretable = scoretable.next_sibling( 'td' )
                        except:
                            break
                        pass
                    pass
                row = row.next_sibling()
                pass
            pass
        except:
            raise
            pass
        #print elem.next_sibling().next_sibling()
        return
        #print self.tables
        sys.exit()
        """Interpreting information. This method must be overridden."""
        raise TypeError( "not implemented" )

    def getIgnorableTags( self ):
        return [ 'span', 'b' ]

    def getStartKeyword( self ):
        return "<!-- from content-start-rail-none.html -->"

    def getEndKeyword( self ):
        return "<!-- from content-end-rail-none.html -->"

    def toHTML( self ):
        """Format data into HTML.
        This method must be overridden."""
        html  = self.html_header()
        for table in self.tables:
            html += '<span class="category">%s</span>\n' % table[ 'tournament' ]
            html += '<table><tr><th colspan="%s">%s</th></tr>\n' % ( table[ 'games' ] + 1, table[ 'gamestat' ] )
            html += '<tr><td class="player"></td>'
            for i in range( 0, table[ 'games' ] ): html += '<td class="game">%d</td>' % ( i + 1 )
            html += '</tr>'
            html += '<tr><td class="player">%s</td>' % table[ 'player1' ]
            for g in table[ 'game1' ]: html += '<td>%s</td>' % g
            html += '</tr>'
            html += '<tr><td class="player">%s</td>' % table[ 'player2' ]
            for g in table[ 'game1' ]: html += '<td>%s</td>' % g
            html += '</tr>'

            html += '</table>\n'
            pass
        html += self.html_footer()
        return html

    def html_css_plugin( self ):
        return """
        <style =\"text/css\">
        table { margin-bottom:20px; }
        table tr td { height:12px; }
        .player { font-weigth:bold; width:80px; }
        .game { width:40px; font-weight:bold; }
        </style>
        """
        """This method is overidable by subclasses which uses stylesheet template and add some features in css."""
        return ""

    pass


if __name__ == "__main__":
    
    import sys
    import tempfile
    from dom import *

    skin = None
    filename_input = None

    obj = newInstance()
    if len( sys.argv ) > 1: filename_input = sys.argv[ 1 ]
    
    if obj == None or obj.isAvailable() == False:
        print "invalid object"
        sys.exit()

    url = obj.getURL()
    encoding = obj.getEncoding()
    if filename_input:
        data = ""
        for line in open( filename_input ).readlines():
            data += line
            pass
        pass
    else:
        import loader
        data = loader.loadHTML( url, encoding )
        pass
    kw_start = obj.getStartKeyword()
    kw_end   = obj.getEndKeyword()
    parser = DOMConverter()
    for tag in obj.getIgnorableTags():
        parser.add_ignoring_tag( tag )
        pass
    if kw_start == None or kw_end == None:
        if encoding == None:
            parser.feed( obj.decode( obj.applyFilter( data ) ) )
        else:
            for line in data.split( "\n" ):
                parser.feed( obj.applyFilter( line ) )
                pass
            pass
        pass
    else:
        flag_inside = False
        parser.feed( '<html>' )
        for line in re.split( "\n", data ):
            line = obj.applyFilter( line )
            if encoding == None:
                line = obj.decode( line.strip() )
                pass
            if flag_inside:
                if line.count( kw_end ) > 0:
                    flag_inside = False
                    break
                else:
                    parser.feed( line )
                    pass
            elif line.count( kw_start ) > 0: flag_inside = True
            pass
        parser.feed( '</html>' )
        pass
    document = parser.get_document()
    obj.interpreteDocument( document )
    contents = obj.toHTML()
    print contents
